// Copyright 2014 The gocui Authors. All rights reserved.
// Use of this source code is governed by a BSD-style
// license that can be found in the LICENSE file.

package main

import (
	"os"
	"runtime/debug"

	"go.wit.com/log"
	"go.wit.com/toolkits/tree"
)

// sets defaults and establishes communication
// to this toolkit from the wit/gui golang package
func init() {
	log.Log(INFO, "Init() of awesome-gocui")

	// init the config struct default values
	Set(&me, "default")

	// Set(&me, "dense")

	me.myTree = tree.New()
	me.myTree.PluginName = "gocui"
	me.myTree.ActionFromChannel = action

	// pluginChan = make(chan widget.Action)

	log.Log(NOW, "Init() start pluginChan")
	// go catchActionChannel()
	log.Sleep(.1) // probably not needed, but in here for now under development
	go mainGogui()
	log.Sleep(.1) // probably not needed, but in here for now under development
}

func standardExit() {
	log.Log(NOW, "standardExit() doing baseGui.Close()")
	me.baseGui.Close()
	log.Log(NOW, "standardExit() doing outf.Close()")
	outf.Close()
	// log(true, "standardExit() setOutput(os.Stdout)")
	// setOutput(os.Stdout)
	log.Log(NOW, "standardExit() send back Quit()")
	// go sendBackQuit() // don't stall here in case the
	// induces a delay in case the callback channel is broken
	log.Sleep(1)
	log.Log(NOW, "standardExit() exit()")
	os.Exit(0)
}

func standardClose() {
	log.Log(NOW, "standardExit() doing baseGui.Close()")
	me.baseGui.Close()
	log.Log(NOW, "standardExit() doing outf.Close()")
	outf.Close()
	os.Stdin = os.Stdin
	os.Stdout = os.Stdout
	os.Stderr = os.Stderr
	log.Log(NOW, "standardExit() send back Quit()")
}

var outf *os.File

func main() {
}

func mainGogui() {
	defer func() {
		if r := recover(); r != nil {
			log.Warn("YAHOOOO Recovered in guiMain application:", r)
			log.Warn("Recovered from panic:", r)
			me.baseGui.Close()
			log.Sleep(1)

			me.myTree.SendToolkitPanic()
			log.Warn("Stack trace:")
			debug.PrintStack()
			panic("BUMMER")
			return
		}
	}()
	var err error
	log.Log(INFO, "main() start Init()")

	outf, err = os.OpenFile("/tmp/witgui.log", os.O_RDWR|os.O_CREATE|os.O_APPEND, 0666)
	if err != nil {
		log.Error(err, "error opening file: %v")
		os.Exit(0)
	}
	os.Stdout = outf
	defer outf.Close()

	// setOutput(outf)
	// log("This is a test log entry")

	ferr, _ := os.OpenFile("/tmp/witgui.err", os.O_APPEND|os.O_CREATE|os.O_WRONLY, 0664)
	os.Stderr = ferr
	gocuiMain()
}
